'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _cluster = require('cluster');

var _cluster2 = _interopRequireDefault(_cluster);

var _queueFactory = require('./queue-factory');

var _queueFactory2 = _interopRequireDefault(_queueFactory);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

exports.default = function (masterFunction, options) {
    var numberOfWorkers = options.numberOfWorkers;
    var workerRespawnDelay = options.workerRespawnDelay;
    var workerKillTimeout = options.workerKillTimeout;
    var log = options.log;


    var queue = (0, _queueFactory2.default)(workerRespawnDelay);

    /**
     * worker.id -> timeout id for killing it
     */
    var crashingWorkers = new Map();

    var messageHandler = function messageHandler(worker) {
        return function (message) {
            log.debug('message "' + message + '" from worker ' + worker.id);
            if (message === 'crashing') {
                log.warn('Worker ' + worker.id + ' with PID ' + worker.process.pid + ' is crashing. Disconnecting it...');
                crashingWorkers.set(worker.id, setTimeout(function () {
                    log.warn('Worker ' + worker.id + ' with PID ' + worker.process.pid + ' hasn\'t disconnected after ' + workerKillTimeout + ' ms. Killing it...');
                    worker.kill();
                }, workerKillTimeout));
                worker.disconnect();
                forkNewWorker();
            }
        };
    };

    var forkNewWorker = function forkNewWorker() {
        return queue.push(forkNewWorkerNow);
    };
    var forkNewWorkerNow = function forkNewWorkerNow() {
        log('Forking a new worker...');
        var worker = _cluster2.default.fork();
        worker.on('message', messageHandler(worker));
    };

    _cluster2.default.on('disconnect', function (worker) {
        log('Worker ' + worker.id + ' with PID ' + worker.process.pid + ' disconnected.');
    });

    _cluster2.default.on('exit', function (worker, code, signal) {
        log('Worker ' + worker.id + ' with PID ' + worker.process.pid + ' exited with code ' + code + ' because of signal ' + signal);
        if (crashingWorkers.has(worker.id)) {
            clearTimeout(crashingWorkers.get(worker.id));
            crashingWorkers.delete(worker.id);
            // already forked when putting it in crashingWorkers, so not forking another one.
        } else {
                forkNewWorker();
            }
    });

    var forkInitialWorkers = function forkInitialWorkers() {
        for (var i = 0; i < numberOfWorkers; i++) {
            forkNewWorkerNow();
        }
    };

    if (typeof masterFunction === 'function') {
        var masterResult = masterFunction(options);
        if (masterResult && typeof masterResult.then === 'function') {
            masterResult.then(forkInitialWorkers).catch(function (reason) {
                log.fatal(reason);
                process.exit(1);
            });
        } else {
            forkInitialWorkers();
        }
    } else {
        forkInitialWorkers();
    }
};

module.exports = exports['default'];